package Common;

=head1 NAME

Common - Common Sysmon Variables and Functions.

=head1 SYNOPSIS

  use lib "lib"; # we assume the application directory is the working directory
  use NBK::Common;

  Common::nbkEvent(<message>);
  Common::nbkCount(<event>,<inc>);
  
  ($NBKVER,$NBKSYS,$NBKAPPDIR)=Common::nbkEnv();
          
=head1 DESCRIPTION

This module provides variables and routines which are common to many
of the Perl scripts in the Unix System Monitor Kit.  The nbkEvent function
provides a common interface for passing events to nbklogw via the event.msg
file.  The nbkCount function provides a common interface for passing event
counter updates to nbklogw via the event.msg pipe.

=cut

$NBKVER='0.8.13';
$NBKAPPDIR=`pwd`;
chomp($NBKAPPDIR);
    
sub nbkEnv{
  return(($NBKVER,'',$NBKAPPDIR));
  }

$NBKMSG="$NBKAPPDIR/log/event.msg";
  
sub nbkEvent{
  ($msg) = @_;
  if(!open(MSG,">>$NBKMSG")){
    print("NBK000E Unable to open message file '$NBKMSG' for append.");
    return(1);
    }
  flock(MSG,2); # lock exclusive 
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time());
  if($year<99){$year=2000+$year}
  else{$year=1900+$year}
  $mon++;
  $ppid=getppid();
  $user=getpwuid($<);
  $date=sprintf("%4d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",$year,$mon,$mday,$hour,$min,$sec);
  print(MSG "event:$date $ppid.$user $msg\n");
  close(MSG);
  flock(MSG,8); # unlock
  return(0);
  }
  
sub nbkCount{
  ($event,$inc) = @_;
  if(!open(MSG,">>$NBKMSG")){
    print("NBK000E Unable to open message file '$NBKMSG' for append.");
    return(1);
    }
  flock(MSG,2); # lock exclusive  
  print(MSG "count:$event:$inc\n");
  close(MSG);
  flock(MSG,8); # unlock
  return(0);
  }  
         
1
