#!/usr/bin/perl
# 2011-03-26 eat - added onclick to image input for compatibility with FireFox 4.0 and IE

package Folder;
use Carp;
use NBK::Planner::Planner;
use NBK::Config;

#===========================================================================================================
# Scheme Methods

# Create new object

sub new{
  my ($package,$planner,$path,$doc)=@_;
  bless {PLANNER => $planner,DOC => $doc, PATH => $path } => $package;
  }

sub deletePlan{
  my ($self,$filePath,$planName)=@_;
  my $planner=$self->{PLANNER};
  #print("Delete request for '$planName' plan/$pathDir/$planName<br>\n");
  # get user's permission on each individual file we operate on
  if(-d "plan/$planName"){
    return("Unable to remove directory $filePath.") if(!rmdir($filePath));                          
    }
  system("rm -fr plan/$planName");
  return("Unable to delete plan/$planName/$planName.xml") if($?);  # remove execution plan
  return('');
  }

# Null XML method because folders don't have any special elements

sub xml{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  return('<!-- Folder has no special XML -->'); # Return XML so caller knows life is good
  }

# Folder transaction handler
#   Return message string

sub transact{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  my $planner=$self->{PLANNER};
  my $doc=$self->{DOC};
  my $planPath=$self->{PATH};
  my $msg='';
  my ($pathDir,$pathElement)=split(/:/,$path);
  ($pathDir,$pathElement)=($1,$2) if(!$pathElement && $pathDir=~/^(.*?)\.(.*)$/);
  my $plan=$pathDir;
  $plan=$2 if($plan=~/(.*?\/)+([^\/]+)$/);
  my $model=$planner->getParam('model');
  if($action eq 'Add'){
    return("User not authorized to add plans to this directory.") if(!$canEdit);
    #my $selectName=$planner->getParam('selectName');
    #$selectName=$model if($action eq 'Add');
    my $planName=$planner->getParam('name');
    return("Plan name required. Use browser back button and submit again with plan name.") if(!$planName);
    my $path="plan/$planName/$planName";
    my $planTitle=$planner->getParam('title');
    return("Plan title required. Use browser back button and submit again with plan title.") if(!$planTitle);
    $planTitle=$planner->encode($planTitle);
    if(-e "$path~.xml" || -e "$path.xml"){
      return("Plan already exists with this name. Use browser back button and submit with new name.");
      }
    my $planPurpose=$planner->getParam('purpose');
    return("Plan purpose required. Use browser back button and submit again with plan purpose.") if(!$planPurpose);
    $planPurpose=join("\n",split(/\r?\n/,$planPurpose));
    $planPurpose=$planner->encode($planPurpose);
    if($model eq '_Folder'){
      print("testing $path<br>\n");
      return("Directory or file exists with this name. Use browser back button and submit with new name.") if(-e "$path"); 
      }
    mkdir("plan/$planName");

    my $planState=$planner->getParam('state');
    my $planReason=$planner->getParam('reason');
    $planReason=$planner->encode($planReason);

    # Set state
    if($planState eq 'enabled'){unlink("plan/$planName/$planName.disabled");}
    elsif($planState eq 'disabled'){
      if(!open(FILE,">plan/$planName/$planName.disabled")){
        return("Unable to mark as disabled");
        } 
      close(FILE);
      }
    
    my $creationTime=time();
    my $userId=$planner->getUserId();

    # Write execution plan - copy selected plan
    open(XML,">$path.xml")|| return("Unable to open file for output. Contact application administrator.");
    open(MODEL,"plan/$model/$model.xml")||return("Unable to open model plan.");
    while(<MODEL>){
      $line=$_;
      last if($line=~/^<description /);
      if($line=~/^<plan/){
        $planType=$1 if(/scheme='(.*?)'/);
        $planType=$1 if(/type='(.*?)'/);
        $planSchema=$1 if(/schema='(.*?)'/);
        }
      print(XML);
      }
    print(XML "<description name='$planName' title='$planTitle'>\n");
    print(XML "$planPurpose\n");
    print(XML "</description>\n");
    while(<MODEL>){
      last if(/^<\/description>/);
      }
    while(<MODEL>){
      last if(/^<(history|\/plan)>/);
      print(XML);
      }
    close(MODEL);
    print(XML "<history>\n");
    print(XML "<change id='1' time='$creationTime' user='$userId' reason='$planReason' update='created plan'/>\n");
    print(XML "</history>\n");
    print(XML "</plan>\n");
    close(XML);

    # Write control plan
    open(XML,">$path~.xml")|| return("Unable to open file for output. Contact application administrator.");
    print(XML "<?xml version='1.0'?>\n");
    print(XML "<plan scheme='Control' version='0.8'>\n");
    print(XML "<description name='$planName' title='$planTitle'>\n");
    print(XML "$planPurpose\n");
    print(XML "</description>\n");
    print(XML "<control scheme='$planType' schema='$planSchema'>\n");
    print(XML "<permits>\n");
    print(XML "<permit user='$userId' role='owner'/>\n");
    print(XML "</permits>\n");
    print(XML "</control>\n");
    print(XML "<history>\n");
    print(XML "<change id='1' time='$creationTime' user='$userId' reason='$planReason' update='created plan'/>\n");
    print(XML "</history>\n");
    print(XML "</plan>\n");
    close(XML);
    system("touch plan/$plan/$planName.child");
    return("Unable to touch folder child file. Contact application administrator.") if($?);
    system("touch plan/$planName/$plan.parent");
    return("Unable to touch folder parent file. Contact application administrator.") if($?);
    $planner->postChange('PlanFolderEntryAdd',$path,"($planName)");
    return(''); 
    }
  elsif($action eq 'Update'){
    my $planState=$planner->getParam('state');
    my $planName=$planner->getParam('name');
    my $model=$planner->getParam('model'); # orignal plan name before edit
    my $planTitle=$planner->getParam('title');
    $planTitle=$planner->encode($planTitle);
    my $planPurpose=$planner->getParam('purpose');
    $planPurpose=join("\n",split(/\r?\n/,$planPurpose));
    $planPurpose=$planner->encode($planPurpose);
    # make sure we have resent backups
    my $baktime=int(time()/(30*60));
    $baktime*=30*60;
    system("cp plan/$model/$model~.xml plan/$model/$model~.xml.$baktime") if(!(-f "plan/$model/$model~.xml.$baktime"));
    system("cp plan/$model/$model.xml plan/$model/$model.xml.$baktime") if(!(-f "plan/$model/$model.xml.$baktime"));
    # handle rename if renaming 
    if($planName ne $model){  # renaming
      return("Selected plan name is already taken - use back button and try a different name") if(-d "plan/$planName");
      my $msg=Config::renamePlan($model,$planName);
      return($msg) if($msg);
      }
    # Get control plan
    open(XML,"plan/$planName/$planName~.xml")||return("Unable to read control plan");
    my $cntlXml='';
    while(<XML>){
      $line=$_;
      last if(/^<description /);
      $cntlXml.=$line;
      }
    while(<XML>){
      last if(/^<\/description>/);
      }
    $cntlXml.="<description name='$planName' title='$planTitle'>\n$planPurpose\n</description>\n";
    $cntlXml.=$_ while(<XML>);
    close(XML);
    # Get execution plan
    open(XML,"plan/$planName/$planName.xml")||return("Unable to read execution plan");
    my $execXml='';
    while(<XML>){
      $line=$_;
      last if(/^<description /);
      $execXml.=$line;
      }
    $execXml.="<description name='$planName' title='$planTitle'>\n$planPurpose\n</description>\n";
    while(<XML>){
      last if(/^<\/description>/);
      }
    $execXml.=$_ while(<XML>);
    close(XML);
    open(XML,">plan/$planName/$planName~.xml")||return("Unable to open control plan for output.");
    print(XML $cntlXml);
    close(XML);
    open(XML,">plan/$planName/$planName.xml")||return("Unable to open execution plan for output.");
    print(XML $execXml);
    close(XML);
    $planner->postChange('PlanFolderEntryRename',$path,"($model to $planName)") if($planName ne $model);
    $planner->postChange('PlanFolderEntryUpdate',$path,"($planName)");
    return('');
    }
  return("Logic error - Folder action '$action' not recognized.");
  }

# Return HTML representation

sub htmlEntry{
  my ($action,$relation,$row,$planner,$planPath,$entryName,$selectName)=@_;
  my %entry;
  my $msg=$planner->getCntl($entryName,\%entry);
  print("$msg<br>\n") if($msg);
  my $entryScheme=$entry{scheme};
  my $html='';
  my $class='';
  my $plan=$planPath;
  $plan=$2 if($planPath=~/(.*?\/)+([^\/]+)$/);
  my $stateImage='planUnilateral.gif';
  my $checkboxName="disable$relation";
  if($relation eq 'Self'){
    if(-f "plan/$entryName/$entryName.disabled"){
      $stateImage='planDisabled.gif';
      $checkboxName='enableSelf';
      }
    elsif(-d "plan/$entryName"){
      $stateImage='planEnabled.gif';
      $checkboxName='disableSelf';
      }
    else{
      $stateImage='planUnilateral.gif';
      $checkboxName='resolveSelf';
      }
    }
  else{
    my $reflection='parent';
    $reflection='child' if($relation eq 'Parent');
    if(-f "plan/$plan/$entryName.disabled"){
      $stateImage='planDisabled.gif';
      $checkboxName="enable$relation";
      }
    elsif(-f "plan/$entryName/$plan.disabled" || -f "plan/$entryName/$entryName.disabled"){
      $stateImage='planDisputed.gif';
      $checkboxName="resolve$relation";
      }
    elsif(-f "plan/$entryName/$plan.$reflection"){
      $stateImage='planEnabled.gif';
      $checkboxName="disable$relation";
      }
    elsif(-f "plan/$entryName/"){
      $checkboxName="resolve$relation";
      }
    }
  my $schemaImage='plan.gif';
  my $schema=$entry{schema};
  $schemaImage='plan'.ucfirst($schema).'.gif';
  my $entryImage='plan'.$entryScheme.'.gif';
  #$entryImage='plan.gif' if(!(-f "$entryImage"));
  $entryImage='plan.gif' if(! -f "web/planner/$entryImage");
  if($row && $row%2){$class='even';}
  elsif($row){$class='odd';}
  else{$class='marker';}
  $class='' if($entryName eq $selectName);
  if($row){$html.="<tr class=$class><td>$row</td><td>";}
  else{
    $html.="<tr class=$class><th>";
    #$html.="<a href=''><img src='planCut.gif' border=0></a>" if($action eq 'Edit');
    $html.="</th><td>";
    }
  $html.="<a href='Planner.cgi?request=Edit;path=$planPath:Folder.entry($entryName)' title='Select to edit plan'><img src='$stateImage' border=0></a>";
  $html.="<input type=checkbox name='$checkboxName' value='$entryName'>" if($action eq 'Edit');
  my $entryDesc=$planner->encode($entry{purpose});
  my $prefix='';
  my $suffix='';
  if($relation eq 'Self'){
    $prefix='<b>';
    $suffix='</b>';
    }
  $html.="</td><td><table cellspacing=0 cellpadding=0><tr><td><img src='$schemaImage' border=0></td><td>&nbsp;<a href='Planner.cgi?request=Display;path=$planPath/$entryName' title='$entry{title}'>$prefix$entryName$suffix</a></td></tr></table></td><td><table cellpadding=0 cellspacing=0><tr><td><img src='$entryImage' border=0></td><td>&nbsp;$entryScheme</td></tr></table></td><td>$prefix$entry{title}$suffix</td><td>$entryDesc</td><td>$entry{admins}</td></tr>\n";
  return($html);
  }

sub html{
  my ($self,$section,$element,$action,$canEdit,$canCntl)=@_;
  my $planner=$self->{PLANNER};
  my $doc=$self->{DOC};
  my $planPath=$self->{PATH};
  my $selectName='';
  $action='Display' if(!($element=~/^Folder/));
  $selectName=$1 if($element=~/^Folder.entry\((.*?)\)/);
  my $html='';
  if(!$section){
    $section="$planPath.Folder.entries"; # default section
    $element='Folder.entries';
    }
  $element='Folder.entries' if(!$element);
  $action='Display' if(!$action);  # default action
  my $folderPath=$section;
  $folderPath=$1 if($folderPath=~/^(.*?)\./);
  my $folderRequest='Display';
  if($action ne 'Edit'){
    $folderPath.='.Folder';
    $folderRequest='Edit';
    }
  $html.="<table width='100%' border=1 cellspacing=0 cellpadding=0>\n";
  #$html.="<tr><th><a class=th href='Planner.cgi?request=$folderRequest;path=$folderPath'><img src='planFolder.gif' border=0></a> Folder</th></tr>\n";
  $html.="<tr><td>\n";
  if($action eq 'Edit' && $canEdit){
    my $entryName='';
    my $entryTitle='';
    my $entryPurpose='';
    my $entryAdmins='';
    my $entryScheme='';
    my $entryState='enabled';  
    $selectName='' if(! -e "plan/$selectName/$selectName.xml");
    ($entryName,$entryTitle,$entryPurpose,$entryScheme,$entryState,$entryAdmins)=$planner->getPlanParts("$planPath/$selectName") if($selectName);
    $html.="<form style='margin-bottom:0;' name=FolderEdit action='Planner.cgi' method=post>\n";
    $html.="<input type='hidden' name='request' value='Update'/>\n";
    $html.="<input type='hidden' name='action' value=''/>\n";
    $html.="<input type='hidden' name='selectName' value='$selectName'/>\n";
    my $folderElement='Folder.entries';
    $folderElement="Folder.entry($selectName)" if($entryName);
    $html.="<input type=hidden name=path value='$planPath:$folderElement'>\n";
    $html.="<table width='100%' class='fill'><tr><th>Plan</th><th colspan=2>";
    $html.="<select name='state'>\n";
    foreach my $state ('enabled','disabled'){
      my $selected='';
      $selected=' SELECTED' if($state eq $entryState);
      $html.="<option value='$state'$selected>$state</option>\n";
      }
    $html.="</select>\n";
    $html.="&nbsp; <input type=image src='planNew.gif' name='image' onclick=\"this.form.action.value='Add'\" title='Create new plan from model'/>\n";
    if($entryName){
      $html.="&nbsp; <input type=image src='planUpdate.gif' title='Write changes to selected plan' name='image' onclick=\"this.form.action.value='Update'\"/>\n"; # only allow update if we found the plan
      }
    else{
      $html.="&nbsp; &nbsp; <select name='relative'>\n<option value='' SELECTED></option>\n";
      if(opendir(DIR,"plan")){
        my @plan=readdir(DIR);
        closedir(DIR);
        foreach my $plan (sort {lc($a) cmp lc($b)} @plan){
          next if($plan=~/^\./);
          $html.="<option value='$plan'>$plan</option>\n" if(-d "plan/$plan");
          }
        }
      $html.="</select>\n";
      $html.="&nbsp; <input type=image src='planChild.gif' name='image' onclick=\"this.form.action.value='AddChild'\" title='Make existing plan a child'/>\n";
      $html.="&nbsp; <input type=image src='planParent.gif' name='image' onclick=\"this.form.action.value='AddParent'\" title='Make existing plan a parent'/>\n";
      }
    $html.="</td></tr>\n";
    $html.="<tr><td><b>Name</b></td><td width='10%'><input type=text name=name value='$entryName' size=80></td><td>Plan name</td></tr>\n";
    $html.="<tr><td><b>Title</b></td><td><input type=text name=title value='$entryTitle' size=80></td><td>Short descrptive title</td></tr>\n";
    $html.="<tr><td><b>Purpose</b></td><td><textarea name=purpose value='$entryPurpose' rows=3 cols=77>$entryPurpose</textarea></td><td>Short description of the plan </td></tr>\n";
    $html.="<tr><td width='5%'><b>Model</b></td><td>";
    if($entryName){
      $html.="<input type=hidden name='model' value='$entryName'>\n";
      $html.="<select name='notmodel' disabled>\n";
      $html.="<option value='$entryName' selected>$entryName</option>\n";
      }
    else{
      $html.="<select name='model'>\n";
      if(opendir(DIR,"plan/_Model")){
        my @model=readdir(DIR);
        close(DIR);
        foreach my $model (sort {lc($a) cmp lc($b)} @model){
          next if(!($model=~/^(.*?)\.child$/));
          $model=$1;
          $html.="<option value='$model'>$model</option>\n";
          }
        }
      }
    $html.="</select></td><td>Plan providing content</td></tr>\n";
    $html.="<tr><th>Reason</th><th colspan=2><input type=text name=reason size=80> Change request number and/or short explanation.</th></tr>\n";
    $html.="</table>\n";
    }
  $html.="<table width='100%' border=0 cellpadding=3 cellspacing=1>\n";
  $html.="<tr><th></th>\n";
  if($action eq 'Edit'){
    $html.="<th width='37'>\n";
    $html.="<input type=image src='planToggle.gif' name='image' onclick=\"this.form.action.value='ToggleRelative'\" title='Toggle state of selected rows'/>\n";
    $html.="<input type=image src='planCut.gif' name='image' onclick=\"this.form.action.value='DeleteRelative'\" title='Delete selected rows'/></th>\n";
    }
  else{$html.="<th width='19'></th>\n";}
  $html.="<th>Plan</th><th>Scheme</th><th>Title</th><th>Purpose</th><th width='20%'>Administrators</th><tr>\n";
  foreach my $relation ('Parent','Self','Child'){
    my $row=0;
    my $plan=$planPath;
    $plan=$2 if($planPath=~/(.*?\/)+([^\/]+)$/);
    if($relation eq 'Self'){
      $html.=htmlEntry($action,$relation,$row,$planner,$planPath,$plan,$selectName);
      next;
      }
    my $dir="plan/$plan";
    opendir(DIR,$dir)||return("Unable to open $dir");
    my @file=readdir(DIR);
    closedir(DIR);
    # NOTE: We should change this to use the getParents and getChildren functions
    foreach my $file (sort @file){
      next if(!($file=~/^(.*?)\.(child|parent)$/));
      $entryName=$1;
      my $rel=$2;
      next if($rel ne lc($relation));
      $row++;
      $html.=htmlEntry($action,$relation,$row,$planner,$planPath,$entryName,$selectName);
      }
    }
  #$html.="<tr><th colspan=7 align=center style='font-size: 5px'></th></tr>\n";
  $html.="</table>\n";
  $html.="</td></tr></table>\n";
  $html.="</form>" if($action eq 'Edit' && $canEdit);
  return($html);
  }

# Generate NodeBrain code - just comments

sub nb{
  my ($self)=@_;
  my $planner=$self->{PLANNER};
  my $doc=$self->{DOC};
  my $nb;
  my $planPath=$self->{PATH};
  my $plan=$planPath;
  $plan=$2 if($planPath=~/(.*?\/)+([^\/]+)$/);
  my $dir="plan/$plan";

  opendir(DIR,$dir)||return("# Directory failed to open","Unable to open $dir");
  my @file=readdir(DIR);
  closedir(DIR);
  foreach my $relation ('Parent','Child'){
    my $entryName;
    my %entry;
    my $nameLen=0;
    my $entryNameLen=0;
    $nb.="#\n# $relation Plans\n";
    foreach $file (sort @file){
      next if(!($file=~/^(.*?)\.(parent|child)$/));
      next if($2 ne lc($relation));
      $entryName=$1;
      ($entryNameInternal,$entryTitle,$entryDesc,$type,$entryState,$entryAdmins)=$planner->getPlanParts("$planPath/$entryName");
      $entryNameLen=length($entryNameInternal);
      $nameLen=$entryNameLen if($entryNameLen>$nameLen);
      $entry{$entryNameInternal}=$entryTitle." [$type]";
      }
    my $row=0;
    foreach $entryName (sort {lc($a) cmp lc($b)} keys %entry){
      $row++;
      my $rowStr=sprintf("%3.1d)",$row);
      my $entryNamePadded=substr($entryName.'                                                     ',0,$nameLen);
      $nb.="# $rowStr $entryNamePadded - $entry{$entryName}\n";
      }
    }
  return($nb);
  }

1;
