#!/usr/bin/perl
# 2011-03-26 eat - added onclick to image input for compatibility with FireFox 4.0 and IE

# This class is used to manage a set of attribute values.  It uses the Attributes.pm class
# to manage the attribute definitions.  Here we are only fussing the values.
#

package Options;
use Carp;
use NBK::Planner::Planner;
use NBK::Planner::Attributes;

#===========================================================================================================
# Scheme Methods

# Create new object

sub new{
  my ($package,$planner,$doc)=@_;
  bless {PLANNER => $planner, DOC => $doc} => $package;
  }

# Convert a list document into XML text
#
# Returns string that is XML if it starts with '<', otherwise an error message

sub xml{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  my $planner=$self->{PLANNER};
  my $optionsDoc=$self->{DOC};
  my $attributesDoc=$optionsDoc->child('attributes');
  $attributesDoc=$optionsDoc->child('columns') if(!$attributesDoc);
  my %column;
  my $xml="<options>\n";
  my $columnsElement='columns';  # switch if element is settings
  $columnsElement=$1 if($element=~/^options\.(column.*)$/);
  my $columnsObj=new Attributes($planner,$attributesDoc);
  my $columnsXml=$columnsObj->xml($plan,$columnsElement,$action,$canEdit,$canCntl);
  return($columnsXml) if(!($columnsXml=~/^</));
  $xml.=$columnsXml;
  $xml.="</options>\n";
  return($xml);  
  }

# Return HTML representation
#
#   Path  - portion of section consumed by parent element
#           This is required for constructing URL's to get
#           back for update. 
#
#   Action  Element
#   ------  --------
#
#   'View'  columns
#           column(<colId>)
#
#   'Edit'  columns
#           column(<colId>)
#   
sub html{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  my $planner=$self->{PLANNER};
  my $optionsDoc=$self->{DOC};
  my $attributesDoc=$optionsDoc->child('attributes');
  $attributesDoc=$optionsDoc->child('columns') if(!$attributesDoc);
  $columnId=$1 if($element=~/^column\((\d+)\)$/);
  $action='Display' if(!$action);  # default action
  $element='options.settings' if(!$element);
  my $html='';
  my %column;
  my $colnum=0;
  # support both new and old element names
  foreach my $elementName ('attribute','column'){
    foreach my $column ($attributesDoc->children($elementName)){
      next if(!$column);
      $column{$column->attribute('id')}=$column;
      $colnum++;
      }
    }
  if($colnum==0){  # display columns until we have some
    $element='options.columns';
    $action='Edit' if($action eq 'Display' && $canEdit);
    }
  my $tableSwitchPath="$path.options.columns";
  my $tableSwitchLabel='Settings';
  if($element=~/^options\.column/){
    $tableSwitchPath="$path.options.settings";
    $tableSwitchLabel='Attributes';
    }
  #$html.="<table width='100%' border=1 cellspacing=0 cellpadding=0><tr><th>\n";
  $html.="<tr><th><a href='Planner.cgi?request=Display;path=$tableSwitchPath'><img src='planTable.gif' border=0 title='Switch between rows and attributes'></a>&nbsp;Options $tableSwitchLabel</th></tr>\n" if($element ne 'options');
  #$html.="</th></tr><tr><td>\n";
  #$html.="<tr><td>\n";

  # Let Attributes.pm do it's thing if the element selects them
  if($element=~/^options.(column.*)$/){
    my $columnsElement=$1;
    my $columnsObj=new Attributes($planner,$attributesDoc);
    my $columnsHtml=$columnsObj->html($path.'.options',$columnsElement,$action,$canEdit,$canCntl);
    $html.=$columnsHtml;
    #$html.="</td></tr></table>\n";
    return($html);
    }
  if($action eq 'Edit' && $canEdit){
    $html.="<form style='margin-bottom:0;' name=OptionAttributes action='Planner.cgi' method=post>\n";
    $html.="<input type='hidden' name='request' value='Update'/>\n";
    $html.="<input type='hidden' name='action' value=''/>\n";
    $html.="<input type='hidden' name='path' value='$path.options.settings'/>\n";
    $html.="<table width='100%' class='fill'>";
    $html.="<tr><th>Reason</th><th colspan=2><input type=text name=reason size=80> Change request number and/or short reason for change.</th></tr>\n";
    $html.="</table>\n";
    }

  # List Options
  $html.="<table width='100%' border=0 cellpadding=3 cellspacing=1>\n";
  $html.="<tr><th width='20'></th>";  
  my $updateImageHtml='';
  if($action eq 'Edit' && $canEdit){
    $html.="<th width='37'>";
    $html.="<input type=image src='planToggle.gif' name='image' onclick=\"this.form.action.value='Toggle'\" title='Toggle state of selected attribute'/>\n";
    $html.="<th>Attribute</th>";
    #$html.="<table cellpadding=0 cellspacing=0><tr><th>Value</th><th>&nbsp; &nbsp; ";
    #$html.="<input type=image src='planUpdate.gif' name='image' onclick=\"this.form.action.value='Assign'\" title='Update Settings'/>\n";
    #$html.="</th></tr></table>\n";
    $updateImageHtml="&nbsp; <input type=image src='planUpdate.gif' name='image' onclick=\"this.form.action.value='Assign'\" title='Update Settings'/>";
    }
  else{$html.="<th width='19'></th><th>Attribute</th>";}
  $html.="<th>Title / Value $updateImageHtml</th><th>Revised</th><th>Planner</th><th>Reason</th></tr>\n";
  $row=0;
  foreach my $id (sort {$a<=>$b} keys %column){
    my $colDoc=$column{$id};
    my $state=$column{$id}->attribute('state');
    $colName=$column{$id}->attribute('name');
    $colType=$column{$id}->attribute('type');
    $colFor=$column{$id}->attribute('for');
    #next if($colFor ne 'opt' && $element eq 'options'); # 2007-11-27 eat - experiment
    $colTitle=$column{$id}->attribute('title');
    $colValue=$column{$id}->value;
    $colValue=$column{$id}->attribute('value') if(!$colValue); # support old value method
    $colFormat=$column{$id}->attribute('format'); 
    my $elementTime=$colDoc->attribute('time');
    my $elementDate=$planner->getUserDate($elementTime) if($elementTime);
    my $elementUserId=$colDoc->attribute('user');
    my $elementUserName='';
    if($elementUserId){
      my $userObj=$planner->getUser($elementUserId);
      if($userObj){$elementUserName=$userObj->getName();}
      else{$elementUserName='Guest';}
      }
    #my $elementUserName=$planner->getUserName($elementUserId) if($elementUserId);
    my $elementReason=$colDoc->attribute('reason');
    $class='even';
    $class='odd' if($row%2);
    $class='' if($id eq $columnId);
    $html.="<tr class=$class><td>$id</td>\n";
    my $linkRequest='Display';
    my $linkTitle='Select to highlight this entry';
    if($canCntl){
      $linkRequest='Edit';
      $linkTitle='Select to edit this entry';
      }
    if($state eq 'deleted'){
      $fontOn="<font color='#b0b0b0'>";  # should use style instead
      $fontOff="</font>";
      $stateImage='deleted.gif';
      $checkName='enable';
      }
    elsif($state eq 'disabled'){
      $fontOn="<font color='#b0b0b0'>";
      $fontOff="</font>";
      $stateImage='planDisabled.gif';
      $checkName='enable';
      }
    else{
      $fontOn="";
      $fontOff="";
      $stateImage='planEnabled.gif';
      $checkName='disable';
      }

    #$html.="<td><a href='Planner.cgi?request=$linkRequest;path=$path.options.column($id)'><img src='$stateImage' border=0 title='$linkTitle'></a>";
    $html.="<td><a href='Planner.cgi?request=$linkRequest;path=$path.column($id)'><img src='$stateImage' border=0 title='$linkTitle'></a>";
    $html.="<input type=checkbox name='$checkName' value='$id'>" if($action eq 'Edit' && $canCntl);
    $html.="</td>";
    $html.="<td>$fontOn$colName$fontOff</td>\n";  # remove value column later
    $html.="<td>$fontOn$colTitle$fontOff<br>\n";
    # 2008-12-14 eat included pulldown for menu formats
    if($action eq 'Edit' && $colFormat=~/;/){  # menu format
      $html.="<select name='val$id'>\n";
      my @option=split(/;/,$colFormat);
      foreach my $option (@option){
        my $selected='';
        $selected=' SELECTED' if($option eq $colValue);
        $html.="<option value='$option'$selected>$option</option>\n";
        }
      $html.="</select>\n";
      }
    else{
      my @text=split(/\r?\n/,$colValue);
      if($#text>0 || $colType eq 'code'){
        $colValue=join("\n",@text);
        if($action eq 'Edit' && $canEdit){$html.="<textarea name=val$id rows=10 cols=80>$colValue</textarea>\n";}
        else{$html.="$fontOn<b><pre>$colValue</pre></b>$fontOff\n";}
        }
      else{
        if($action eq 'Edit' && $canEdit){$html.="<input type=text name=val$id value='$colValue' size=100>\n";}
        else{$html.="$fontOn<b>$colValue</b>$fontOff\n";}
        }
      }
    $html.="</td><td class=fill>$elementDate</td><td class=fill>$elementUserName</td><td class=fill>$elementReason</td></tr>\n";
    $row++;
    }
  $html.="</table>";
  #$html.="</td></tr></table>\n";
  $html.="</form>" if($action eq 'Edit' && $canEdit);
  return($html);
  }

sub nb{
  my ($self)=@_;
  my $optionsDoc=$self->{DOC};
  my $attributesDoc=$optionsDoc->child('attributes');
  $attributesDoc=$optionsDoc->child('columns') if(!$attributesDoc);
  my $nb="#\n# Options:\n#\n";
  return($nb) if(!$attributesDoc);
  my %columnElement;
  # support both new and old element names
  foreach my $elementName ('attribute','column'){
    foreach my $columnDoc ($attributesDoc->children($elementName)){
      next if(!$columnDoc);
      $columnId=$columnDoc->attribute('id');
      $columnElement{$columnId}=$columnDoc;      # create hash to order and lookup by id
      }
    }
  foreach my $columnId (sort {$a<=>$b} keys %columnElement){
    $element=$columnElement{$columnId};
    $columnName=$element->attribute('name');
    $columnState=$element->attribute('state');
    $columnFor=$element->attribute('for');
    $columnType=$element->attribute('type');
    $columnTitle=$element->attribute('title');
    $columnValue=$element->attribute('value');
    $columnValue='"'.$columnValue.'"' if($columnType eq 'string');
    my $prefix='';
    $prefix='# ' if($columnState eq 'disabled');
    if($columnFor eq 'doc'){
      $nb.="#$prefix DOC: $columnName=$columnValue; # $columnTitle\n";
      }
    elsif($columnFor eq 'audit'){
      $nb.="#$prefix AUDIT: $columnName=$columnValue; # $columnTitle\n";
      }
    else{
      $nb.="${prefix}assert $columnName=$columnValue; # $columnTitle\n";
      }
    }
  $nb.="#\n";
  return($nb);
  }

1;
