#!/usr/bin/perl
# 2011-03-26 eat - added onclick to image input for compatibility with FireFox 4.0 and IE
# 2013-03-03 eat 0.8.14 Retaining compiler case and attempting compatibility at install time

package Plan;
use Carp;
use NBK::Planner::Planner;
use NBK::Planner::Control;
use NBK::Planner::Folder;
use NBK::Planner::Profile;
use NBK::Planner::Table;

sub new{
  my ($package,$planner,$path)=@_;
  my ($path,$element)=split(/:/,$path);
  ($path,$element)=($1,$2) if(!$element && $path=~/^(.*?)\.(.*)$/);
  my $planName=$path;
  $planName=$2 if($planName=~/^(.*?\/)+([^\/]+)$/);
  my $doc=$planner->loadPlan($path);
  if(!$doc){
    print("Unable to load plan at path $path<br>\n");
    return;
    }
  my $scheme=$doc->child('plan')->attribute('scheme');
  $scheme=$doc->child('plan')->attribute('type') if(!$scheme);
  my $compiler;
  my $schema=$doc->child('plan')->attribute('schema');
  $schema=$scheme if(!$schema);
  $schema=lc($schema) if(!($schema=~/^_/));
  # load the control document for reference later and to get current user's authority relative to this plan
  my $controlDoc=$doc;
  my $controlPath=$path;
  my $basePath=$path;
  if($scheme ne 'Control'){
    $controlPath.='~';
    $controlDoc=$planner->loadPlan($controlPath);
    }
  else{$basePath=$1 if($basePath=~/^(.*)~$/);}
  $control=new Control($planner,$controlPath,$controlDoc);
  
  my $folder=new Folder($planner,$basePath,$doc);
  my $schemaObj;
  if($scheme eq '_Folder'){$schemaObj=$folder;}
  elsif($scheme eq 'Control'){$schemaObj=$control;}
  elsif($scheme eq 'Profile'){$schemaObj=new Profile($planner,$path,$doc);}
  elsif($scheme ne $schema){
    my $schemaDoc=$doc->child('plan')->child($schema);
    if($schema eq 'table'){$schemaObj=new Table($planner,$schemaDoc);}
    else{print("Don't recognize plan schema '$schema'<br>\n");}
    $compiler=$scheme;
    # 2013-03-02 eat - moving away from the lowercase so will try lower case when installing compilers if not found
    #$compiler=lc($compiler) if(!($compiler=~/^_/));
    }
  else{print("Don't recognize plan scheme '$scheme'<br>\n");}
  bless {NAME => $planName, PLANNER => $planner, CONTROL => $control, DOC => $doc, PATH => $path, FOLDER => $folder, SCHEME => $schemaObj, SCHEMA => $schema, COMPILER => $compiler} => $package;
  }

# Generate schemaObj xml from document with updates if action specified
#
# Returns a string that is XML if it starts with '<'.  Otherwise it is an error message.
#   

sub xml{
  my ($self,$path,$element,$action)=@_;
  my $planner=$self->{PLANNER};
  my $doc=$self->{DOC};
  my $schemaObj=$self->{SCHEME};
  my $schema=$self->{SCHEMA};
  my $planDoc=$doc->child('plan');
  my $scheme=$planDoc->attribute('scheme');
  $scheme=$planDoc->attribute('type') if(!$scheme); # support transition
  my $version=$planDoc->attribute('version');
  my $sectionType;
  my $canEdit=$self->{CONTROL}->getAuthorized('Edit',0);
  my $canCntl=$self->{CONTROL}->getAuthorized('Edit',1);

  return("User not autorized to edit this plan.") if(!$canEdit);
  my $requestScheme=$scheme;
  $requestScheme=$1 if($element=~/^(.*?)\./);
  $requestScheme=lc($requestScheme);
  if($requestScheme ne 'folder' && $element ne 'guide' && $requestScheme ne $schema){
    return("Update requested on $requestScheme but document is $scheme - schema=$schema"); 
    }
  my $xml="<?xml version='1.0'?>\n";
  $xml.="<plan scheme='$scheme' schema='$schema' version='0.8'>\n";
  my $planName='';
  my $planTitle=''; 
  my $planPurpose=''; 
  $planDescObj=$planDoc->child('description');
  if($planDescObj){
    $planName=$planDescObj->attribute('name');
    $planTitle=$planDescObj->attribute('title');
    $planPurpose=$planDescObj->value;
    }
  $planTitle=$planner->encode($planTitle);
  $planPurpose=$planner->encode($planPurpose);
  $xml.="<description name='$planName' title='$planTitle'>$planPurpose</description>\n";

  if($element eq 'guide' && $action eq 'Update'){
    my $planGuide=$planner->getParam('guide');
    if($planGuide){
      $xml.="<guide>";
      $xml.=$planner->encode($planGuide);
      $xml.="</guide>\n";
      }
    }
  else{
    my $guideDoc=$planDoc->child('guide');
    if($guideDoc && $guideDoc->value){
      $xml.="<guide>";
      $xml.=$planner->encode($guideDoc->value);
      $xml.="</guide>\n";
      }
    }

  # Let schemaObj module build variable XML
  my $schemaXml=$schemaObj->xml($path,$element,$action,$canEdit,$canCntl);
  return($schemaXml) if(!($schemaXml=~/^</));
  $xml.=$schemaXml;

  # Build XML for history - creates history if not already there
  $xml.="<history>\n";
  my $planHistory=$planDoc->child('history');
  my $newId;
  if($planHistory){
    my %change;
    
    $newId=1;
    foreach $change ($planHistory->children('change')){
      last if(!$change);
      $id=$change->attribute('id');
      $id=$newId if(!$id);           # automatically upgrade older files 
      $change{$id}=$change;
      $newId++;
      }
    $newId=1;
    foreach $id (sort {$a<=>$b} keys %change){
      $change=$change{$id};
      $time=$change->attribute('time'); 
      $utc=$change->attribute('utc'); 
      $time=$utc if($utc);  # convert old entries
      $userId=$change->attribute('user');
      $changeUpdate=$change->attribute('update');
      $changeUpdate=$planner->encode($changeUpdate);
      $changeReason=$change->attribute('reason');
      $changeReason=$planner->encode($changeReason);
      $xml.="<change id='$newId' time='$time' user='$userId' update='$changeUpdate' reason='$changeReason'/>\n";
      #$newId=$id;
      $newId++;
      }
    }
  # Include new entries here
  $time=time(); 
  $userId=$planner->getUserId();
  $changeReason=$planner->getParam('reason');
  $changeReason=$planner->encode($changeReason);
  foreach my $list (@{$planner->{EVENT}}){
    my ($verb,$plan,$text)=@$list;
    $text=$planner->encode($text);
    $xml.="<change id='$newId' time='$time' user='$userId' update='$verb $text' reason='$changeReason'/>\n"; 
    $newId++;
    }
  $xml.="</history>\n";
  $xml.="</plan>\n";
  return($xml);
  }

sub html{
  my ($self,$path,$element,$action)=@_;
  #print("Plan.html called with path=$path element=$element action=$action<br>\n");
  my $folder=$self->{FOLDER};
  my $schemaObj=$self->{SCHEME};
  my $schema=$self->{SCHEMA};
  my $planner=$self->{PLANNER};
  my $planPath=$self->{PATH};
  my $doc=$self->{DOC};
  my $planDoc=$doc->child('plan');
  my $scheme=$planDoc->attribute('scheme');
  $scheme=$planDoc->attribute('type') if(!$scheme);
  my $canEdit=$self->{CONTROL}->getAuthorized('Edit',0);
  my $canCntl=$self->{CONTROL}->getAuthorized('Edit',1);
  $action='Display' if(!$canEdit);
  my $html='';
  $html.="<table width='100%' border=0 cellpadding=0 cellspacing=0><tr><td>\n";
  $html.="<table width='100%' border=0 cellpadding=0 cellspacing=0><tr><th width='60%'>\n";
  $html.="<a class=th href='Planner.cgi'><img src='planHome.gif' border=0 title='Home plan folder'></a>";
  # Construct linked path so user can jump back to a higher level directory
  my ($pathDir,$pathElement)=split(/:/,$path);
  ($pathDir,$pathElement)=($1,$2) if(!$pathElement && $pathDir=~/^(.*?)\.(.*)$/);
  my $planName=$pathDir;
  $planName=$2 if($planName=~/^(.*?\/)+([^\/]+)$/);
  my $baseName=$planName;
  $baseName=$1 if($baseName=~/(.*?)(~|\-)$/);
  my @pathDir=split(/\//,$pathDir);
  my $pathString=shift(@pathDir);
  my $pathLinks="<a href='Planner.cgi?request=Display;path=$pathString'>$pathString</a>";
  my $pathUp=$pathString;
  foreach my $pathDir (@pathDir){
    $pathUp=$pathString;
    $pathString.="/$pathDir";
    $pathLinks.="/<a href='Planner.cgi?request=Display;path=$pathString'>$pathDir</a>";
    }
  $pathLinks.=".$element" if($element);
  $html.="&nbsp;<a href='Planner.cgi?request=Display;path=$pathUp'><img src='planBack.gif' border=0 title='Return to prior plan'></a>";


  my $linkAction=$action;
  $linkAction='Edit' if($action ne 'Display');
  $linkAction='Display' if(!$canEdit);
  $html.="&nbsp;<a class=th href='Planner.cgi?request=$linkAction;path=$path:$element'><img src='planLink.gif' border=0 title='Link to this page'></a>";
  my $planImage='plan.gif';
  $planImage='planFolder.gif' if($scheme eq 'Folder');
  $planImage='planItem.gif'; # experiment
  my $planRequest='Display';
  my $folderPath=$path;
  $folderPath=$1 if($folderPath=~/^(.*)\./);
  if($action eq 'Display' && $canEdit){
    $planRequest='Edit';
    $folderPath.='.Folder';
    $folderImage='planFolder.gif';
    }
  $planTitle='';
  $planPurpose='';
  $planDescObj=$planDoc->child('description');
  if($planDescObj){
    $planInternalName=$planDescObj->attribute('name');
    $planTitle=$planDescObj->attribute('title');
    $planPurpose=$planner->encode($planDescObj->value);
    }


  my $folderImage='planFolder.gif';
  $folderImage='planFolderOpen.gif' if($element=~/^Folder/);
  $html.="&nbsp; &nbsp; &nbsp;<a class=th href='Planner.cgi?request=$planRequest;path=$folderPath'><img src='$folderImage' border=0></a>\n";
  #$html.="<a class=th href='Planner.cgi?request=$planRequest;path=$path:$element'><img src='$planImage' border=0 title='$planRequest plan'></a>\n"; 
  my $basePath=$planPath;
  $basePath=$1 if($planPath=~/^(.*?)~$/);
  if($basePath eq $planPath){
    $html.="<a href='Planner.cgi?request=Display;path=$basePath~;'><img src='planControl.gif' border=0 title='Display control plan'></a>\n";
    }
  else{
    $html.="<a href='Planner.cgi?request=Display;path=$basePath;'><img src='planPlan.gif' border=0 title='Display execution plan'></a>\n";
    }
  $html.="&nbsp; &nbsp; &nbsp;<a href='plan/$baseName/$planName.xml'><img src='planXml.gif' border=0 title='Display XML'></a>\n";
  $html.="<a href='Planner.cgi?request=Code;path=$path:$element'><img src='planCode.gif' border=0 title='Display NodeBrain code'></a>\n" if($schema ne 'folder');
  $html.="<a href='Planner.cgi?request=Config;path=$path:$element'><img src='planConfig.gif' border=0 title='Display NodeBrain kit config file'></a>\n" if($schema eq 'table');
  $html.="<a href='Planner.cgi?request=Table;path=$path:$element'><img src='planTable.gif' border=0 title='Display NodeBrain kit config file'></a>\n" if($schema eq 'table');
  #$html.="&nbsp; &nbsp; &nbsp;<a href='Planner.cgi?request=Help;path=$path:$element'><img src='planHelp.gif' border=0 title='Display Help'></a>\n";
  $html.="&nbsp; &nbsp; &nbsp;<a href=\"javascript:help('planner')\"><img src='planHelp.gif' border=0 title='Display Help'></a>\n";
  $html.="</th>\n";
  my $userName=$planner->getUserName();
  $html.="<th width='40%' align=right><a class=th href='Planner.cgi?request=EditProfile'><img src='planProfile.gif' border=0 title='User profile'></a> $userName</th><tr>\n";
  $html.="</table>\n";
  $html.="<table width='100%' cellspacing=0>\n";
  $html.="<tr class=odd><td>$pathLinks</td></tr>\n";
  $html.="</table>\n";
  $html.="</td></tr></table>\n";

  # Create folder section
  $html.=$folder->html($path,$element,$action,$canEdit,$canCntl);

  # Create the guide section
  $html.="<table width='100%' border=1 cellspacing='0'>\n";
  $html.="<tr><th>";
  if($canCntl && ($element ne 'guide' || $action eq 'Display')){$html.="<a href='Planner.cgi?request=Edit;path=$path.guide;'>\n";}
  else{$html.="<a href='Planner.cgi?request=Display;path=$path;'>\n";}
  $html.="<img src='planGuide.gif' border=0 title='Edit guide'>";
  $html.="</a> Guide</th></tr>\n";
  my $guideDoc=$planDoc->child('guide');
  $html.="<tr><td class='guide'>".$guideDoc->value."</td></tr></table>\n" if($guideDoc);
  if($action eq 'Edit' && $canCntl && $element eq 'guide'){
    my $value=''; 
    $value=$guideDoc->value if($guideDoc);
    $html.="<form style='margin-bottom:0;' name=GuidEdit action='Planner.cgi' method=post>\n";
    $html.="<input type='hidden' name='request' value='Update'/>\n";
    $html.="<input type='hidden' name='action' value=''/>\n";
    $html.="<input type='hidden' name='path' value='$path.guide'/>\n";
    $html.="<table class='fill' border=1 width='100%' cellspacing='0'>\n";
    $html.="<tr><th><input type=image src='planUpdate.gif' title='Write changes to guide' name='image' onclick=\"this.form.action.value='Update'\"/> Guide HTML Editor</th></tr>\n";
    $html.="<tr><td><textarea name=guide rows=60 width='100%'>$value</textarea></td></tr>\n";
    $html.="</table>\n";
    $html.="</form>\n";
    }

  # Create plan specific section
  #print("Plan.html calling schemaObj.html with path=$path element=$element action=$action<br>\n");
  $html.=$schemaObj->html($path,$element,$action,$canEdit,$canCntl) if($schemaObj!=$folder && !($element=~/^Folder/));

  # Build HTML for history
  my $planHistory=$planDoc->child('history');
  if($planHistory){
    my %change;
    my $newId;

    $newId=1;
    foreach $change ($planHistory->children('change')){
      last if(!$change);
      $id=$change->attribute('id');
      $id=$newId if(!$id);           # handle older files that didn't have id
      $change{$id}=$change;
      $newId++;
      }
    my $classCode=0;
    $html.="<table width='100%' border=1 cellspacing=0 cellpadding=0><tr><th> &nbsp; Change History</th></tr><tr class=even><td>\n";
    $html.="<table border=1 cellspacing=0 cellpadding=0><tr><td>\n";
    $html.="<table cellspacing=0 cellpadding=2><tr><th width='140'>Time</th><th>Planner</th><th>Change</th><th>Reason</th></tr>\n";
    foreach $id (sort {$b<=>$a} keys %change){
      $change=$change{$id};
      $time=$change->attribute('time');
      $userTime=$planner->getUserTime($time);
      $userId=$change->attribute('user');
      my $userName=$userId;
      my $adminObj=$planner->getUser($userId);
      $userName=$adminObj->getName(); 
      my $changeUpdate=$change->attribute('update');
      my $changeReason=$change->attribute('reason');
      if(($classCode=($classCode+1)%2)){$class='even';}
      else{$class='odd';}
      $html.="<tr class='$class'><td>$userTime</td><td>$userName</td><td>$changeUpdate</td><td>$changeReason</td></td><tr>\n";
      }
    }
  $html.="</table>\n";
  $html.="</td></tr></table>\n";
  $html.="</td></tr></table>\n";
  return($html);
  }

# Create nb code
#
#   
sub nb{
  my ($self)=@_;
  my $planner=$self->{PLANNER};
  my $planName=$self->{PATH};
  $planName=$2 if($planName=~/(.*?\/)+([^\/]+)$/);
  my $doc=$self->{DOC};
  my $schemaObj=$self->{SCHEME};
  my $compiler=$self->{COMPILER};
  my $nb='';
  $plan=$doc->child('plan');
  return("# NBK000E Expecting PLAN element as root of document $planFile.xml","Plan element not found") if(!$plan);

  %planAttrKnown=(scheme => 1, type => 1, version => 1, schema => 1);
  %planAttr=$plan->attributes;
  foreach $planAttr (keys %planAttr){
    $nb.="# NBK000E PLAN attribute $planAttr is not recognized.\n" if(!$planAttrKnown{$planAttr});
    }
  my $scheme=$plan->attribute('scheme');
  $scheme=$plan->attribute('type') if(!$scheme);
  $nb.="# NBK000E Type attribute required in plan element.\n" if(!$scheme);
  $planVersion=$planAttr{'version'};
  $nb.="# NBK000E Version $planVersion not supported.\n" if($planVersion > 0.8);
  $nb.="# NodeBrain Planner 0.8.0 (XML Schema $planVersion)\n#\n";
  %planDescriptionAttrKnown=(name => 1, title => 1);
  $planDescription=$plan->child('description');
  if(!$planDescription){$nb.="# NBK000E Plan description required\n";}
  else{ 
    %planDescriptionAttr=$planDescription->attributes;
    foreach $planDescriptionAttr (keys %planDescriptionAttr){
      $nb.="# NBK000E Description attribute $planDescriptionAttr is not recognized." if(!$planDescriptionAttrKnown{$planDescriptionAttr});
      }
    # don't use internal name
    my $planDisplayName=$planDescription->attribute('name');
    $nb.="# Name:      $planName\n#\n";
    $planTitle=$planDescription->attribute('title');
    $nb.="# Title:     $planTitle\n#\n";
    $nb.="# Purpose:\n";
    $planPurpose=$planDescription->value;
    @lines=split(/\n/m,$planPurpose);
    foreach $line (@lines){
      $nb.="#   $line\n";
      }
    $nb.="#\n";
    }
  $nb.="# Warning:\n#\n#   Do not edit this file manually unless it is no longer under the control of NodeBrain Planner.\n";
  $nb.="#   Otherwise, your changes will be removed by the next update.\n#\n";
  $nb.="# Path:      CABOODLE/plan/$planName\n#\n";
  $nb.="# Scheme:    $scheme\n#\n";
  if($scheme ne 'Control'){
    $nb.="#================================================================================================\n";
    $nb.=$self->{CONTROL}->nb();
    }
  $nb.="#================================================================================================\n";
  $nb.="# History:\n";
  $nb.="#\n";
  $nb.="# Date       Time     User - Change - Reason\n";
  $nb.="# ---------- -------- ------------------------------------------------------------------------------\n";
  my $planHistory=$doc->child('plan')->child('history');
  if($planHistory){
    my %change;
    foreach my $change ($planHistory->children('change')){
      $change{$change->attribute('id')}=$change if($change);
      }
    foreach my $id (sort {$a<=>$b} keys %change){
      $change=$change{$id};
      $time=$change->attribute('time');
      $timeStr=$planner->getUserTime($time);
      $userId=$change->attribute('user');
      $admin=$planner->getUser($userId);
      $adminName=$admin->getName();
      $changeReason=$change->attribute('reason');
      $changeUpdate=$change->attribute('update');
      $nb.="# $timeStr $adminName - $changeUpdate - $changeReason\n";
      }
    }
  $nb.="#================================================================================================\n";
  if($compiler){
    #if(Config::install($planName,"adapter/$compiler.compiler")!=0){
    #  return($nb,"CAB000E Unable to install adapter/$compiler.compiler");
    #  }
    #my $planNb='';
    #if(!open(NB,"adapter/$compiler.compiler $planName |")){
    #  return($nb,"CAB000E Unable to execute adapter/$compiler.compiler");
    #  }
    #$planNb.=$_ while(<NB>);
    #close(NB);
    #return($nb,$planNb) if(!($planNb=~/^#/));  # Error if doesn't start with '#'
    #$nb.=$planNb;
    }
  else{$nb.=$schemaObj->nb();}
  return($nb);
  }

sub cfg{
  my ($self)=@_;
  my $planner=$self->{PLANNER};
  my $planPath=$self->{PATH};
  my $doc=$self->{DOC};
  my $schemaObj=$self->{SCHEME};
  my $schema=$self->{SCHEMA};
  my $compiler=$self->{COMPILER};
  my $cfg='';
  $plan=$doc->child('plan');
  return("# NBK000E Expecting PLAN element as root of document $planFile.xml","Plan element not found") if(!$plan);

  %planAttrKnown=(scheme => 1, type => 1,version => 1, schema => 1, state =>1);  # type and state are obselete
  %planAttr=$plan->attributes;
  foreach $planAttr (keys %planAttr){
    $cfg.="# NBK000E PLAN attribute $planAttr is not recognized.\n" if(!$planAttrKnown{$planAttr});
    }
  my $scheme=$plan->attribute('scheme');
  $scheme=$plan->attribute('type') if(!$scheme);

  my $planState=$planner->getPlanState($planPath);

  $cfg.="# NBK000E Scheme attribute required in plan element.\n" if(!$scheme);
  $planVersion=$planAttr{'version'};
  $cfg.="# NBK000E Version $planVersion not supported.\n" if($planVersion > 0.8);
  $cfg.="# NodeBrain Planner 0.8.0 (XML Schema $planVersion)\n#\n";

  my $host=`hostname`;
  chomp($host);
  my ($fqdn)=gethostbyname($host);
  $host=$fqdn if($fqdn);
  $cfg.="# Host:      $host\n#\n";

  my $CABOODLE=`pwd`;
  chomp($CABOODLE);
  $cfg.="# Caboodle:  $CABOODLE\n#\n";
  my @levels=split(/\//,$planPath);
  my $planName=pop(@levels);
  $planName=$1 if($planName=~/^(.*?)(:|\.)/);
  $cfg.="# Plan:      $planName\n#\n";
  $cfg.="# Scheme:    $scheme\n#\n";
  if($scheme ne 'Control'){
    $cfg.="#================================================================================================\n";
    $cfg.=$self->{CONTROL}->nb();
    }
  $cfg.="#================================================================================================\n";
  my $folderObj=$self->{FOLDER};
  $cfg.=$folderObj->nb();
  $cfg.="#================================================================================================\n";
  $cfg.="\n.scheme: $scheme\n";
  %planDescriptionAttrKnown=(name => 1, title => 1);
  $planDescription=$plan->child('description');
  if(!$planDescription){$cfg.="# NBK000E Plan description required\n";}
  else{
    %planDescriptionAttr=$planDescription->attributes;
    foreach $planDescriptionAttr (keys %planDescriptionAttr){
      $cfg.="# NBK000E Description attribute $planDescriptionAttr is not recognized." if(!$planDescriptionAttrKnown{$planDescriptionAttr});
      }
    # don't use internal name
    my $planDisplayName=$planDescription->attribute('name');
    $planTitle=$planDescription->attribute('title');
    $cfg.="\n.title: $planTitle\n";
    $cfg.="\n.purpose{";
    $planPurpose=$planDescription->value;
    @lines=split(/\n/m,$planPurpose);
    foreach $line (@lines){
      $cfg.=" $line\n";
      }
    $cfg.="}\n";
    }
  $cfg.="\n.guide{\n";
  my $guideDoc=$plan->child('guide');
  if($guideDoc){
    $planGuide=$guideDoc->value;
    @lines=split(/\n/m,$planGuide);
    shift(@lines) if(!$lines[0]);
    foreach $line (@lines){
      $cfg.=" $line\n";
      }
    }
  $cfg.="}\n\n";
  $cfg.=$schemaObj->cfg() if($schema eq 'table' || $schema eq 'list');
  return($cfg);
  }

sub table{
  my ($self)=@_;
  my $schemaObj=$self->{SCHEME};
  my $schema=$self->{SCHEMA};
  my $table=$schemaObj->table() if($schema eq 'table' || $schema eq 'list');
  return($table);
  }

1;
