#!/usr/bin/perl

# This class is a simple scheme that contains one Options.pm instance and let's it do all the work.
# The purpose of this layer is to define an arrangement of other objects.  Since we only need one
# child object (Options.pm) in this case, there isn't much to do here.
# 

package Profile;
use Carp;
use NBK::Planner::Planner;
use NBK::Planner::Options;

#===========================================================================================================
# Scheme Methods

# Create new object

sub new{
  my ($package,$planner,$path,$doc)=@_;
  my $planDoc=$doc->child('plan');
  my $profileDoc=$planDoc->child('profile');
  bless {PLANNER => $planner, PATH => $path , DOC => $profileDoc } => $package;
  }

# Convert a profile document into XML text
#
# Returns string that is XML if it starts with '<', otherwise and error message

# Plan.pm and all the subordinates should agree to only include in the path, that which is consumed.
# It is currently sending the full path and then schemes have to chop it at ':' and then extend it.
# Schemes are currently passing $path and $element down as two parts of the full path.  That way,
# subordinate classes can use up a part of $element and append the used part to $path.

sub xml{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  my $planner=$self->{PLANNER};
  my $profileDoc=$self->{DOC};
  my $optionsDoc=$profileDoc->child('options');
  my $optionsObj=new Options($planner,$optionsDoc);
  my $xml="<profile>\n";;
  
  return("User does not have edit authority.") if(!$canEdit);

  if($element=~/^Profile.(options.*)$/){
    my $childElement=$1;
    my ($childPath,$nothing)=split(/:/,$path);
    ($childPath,$nothing)=($1,$2) if(!$nothing && $childPath=~/^(.*?)\.(.*)$/);
    $childPath.=':Profile';
    my $childXml=$optionsObj->xml($childPath,$childElement,$action,$canEdit,$canCntl); 
    return($childXml) if(!($childXml=~/^</));
    $xml.=$childXml;
    }
  else{return("Element not understood by Profile.pm");}
  $xml.="</profile>\n";
  return($xml);  
  }

# Return HTML representation of a profile
#
sub html{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  my $planner=$self->{PLANNER};
  my $profileDoc=$self->{DOC};
  my $optionsDoc=$profileDoc->child('options');
  my $optionsObj=new Options($planner,$optionsDoc);
  my $html='';

  # may want an outer table - but leave it up to options for now
  $element='Profile.options.settings' if(!$element); # define
  if($element=~/^Profile.(options.*)$/){
    my $childElement=$1;
    my ($childPath,$nothing)=split(/:/,$path);
    ($childPath,$nothing)=($1,$2) if(!$nothing && $childPath=~/^(.*?)\.(.*)$/);
    $childPath.=':Profile';
    my $childHtml=$optionsObj->html($childPath,$childElement,$action,$canEdit,$canCntl);
    return($childHtml) if(!($childHtml=~/^</));
    $html.=$childHtml;
    }
  else{return("Element not understood by Profile.pm");}
  #$html.="</table>\n";
  return($html);
  }

sub nb{
  my ($self)=@_;
  my $planner=$self->{PLANNER};
  my $profileDoc=$self->{DOC};
  my $optionsDoc=$profileDoc->child('options');
  my $optionsObj=new Options($planner,$optionsDoc);
  my $nb="#\n# Profile:\n#\n";

  $nb.=$optionsObj->nb();
  return($nb);
  }

1;
