#!/usr/bin/perl
#
# File:     Planner.cgi
#
# Title:    NodeBrain Kit Rule Configuration Script (web)
#
# Purpose:
#
#   This script is used to configure NodeBrain Kit rules using
#   a CGI web interface.
#
# Description:
#
#   NodeBrain Kit XML Scheme 1.0 is used by this script to edit plans and generate NodeBrain
#   code.
#
# Parameters:
#
#   request   - Requested action
#
#               none     - display the Diretory plan
#
#               Display  - display a selected plan
#
#               New      - display a form to create a new plan
#
#               Create   - create a plan using the input data.
#
#               Update   - update a plan using the input data.
#
#
#   type      - Type of plan to use in the requested action.
#
#   name      - Name of plan to use in the requested action.
#
#    
#================================================================================================
# Change History
#
# Date       Name/Change
# ---------- ------------------------------------------------------------------------------------
# 2006/06/01 Ed Trettevik - original prototype
# 2007/10/02 eat - modified to store *.nb files in plan directory instead of subdirectory where *.xml file resides
#            This was done to make it easier to look at all the plan *.nb files in one place.
# 2010/02/11 eat 0.7.5 - changed name from nbkPlanner.cgi to Planner.cgi
# 2011-03-13 eat 0.8.0 - updated version
# 2011-10-19 eat 0.8.0 - modified plan index file to provide full relative path within caboodle
#            This means the file can be outside the caboodle also simply by providing a full
#            path.  This is not recommended since multiple caboodles may clobber each other.
# 2011-10-22 eat 0.8.0 - support NB_WEBSTER_CONFIG environment variable for configuration file name
#            This enables multiple Webster ports in the same caboodle to use unique configuration
#            files.
#================================================================================================
use XML::Parser;
use XML::SimpleObject;
use CGI qw(:standard);
use lib "../../lib";
use NBK::Caboodle;
use NBK::Planner::Planner;
use NBK::Planner::Plan;
use NBK::Planner::Folder;
chdir('../..'); # change to caboodle directory

my $cabCopyright;

sub abort{
  ($msg)=@_;
  msg(0,$msg); #print("<p>*** $msg ***<br><br>\n");
  print("<span class=foot><hr>$cabCopyright</span></body></html>");
  exit(0);
  }
  
# Read text from file
sub getText{
  (my $file)=@_;
  my $text='';
  open(FILE,"$file")||abort "Unable to open '$file' for input";
  $text.=$_ while(<FILE>);
  return($text);
  }

# Write text to file
sub putText{
  (my $file,my $text)=@_;
  if(-f $file){
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=gmtime();
    $backupFile=sprintf("$file.%4.4u-%2.2u-%2.2u-gmt.bak",$year+1900,$mon+1,$mday);
    system("cp $file $backupFile") if(!(-f $backupFile));
    }
  open(FILE,">$file")||abort "Unable to open '$file' for output";
  $text=join('',split(/
/,$text));   # get rid of ^M 
  $text=join("\n",split(/\n/,$text)); # clean up text
  print(FILE $text."\n");
  close(FILE);
  }

sub displayFileEditForm{
  (my $file)=@_;
  $text=getText("$file");
  print("<form style='margin-bottom:0;' name=fileEdit action='nbkPlan.cgi' method=post>\n");
  print("<input type=hidden name=table value='$table'>\n");
  print("<input type=hidden name=file value='$file'>\n");
  print("<table cellspacing=0 cellpadding=3><tr><th>\n");
  print("<input type=submit name=request value='Save'>\n") if($admin==2);
  print(" $tableTitle</th></tr><tr class=even><th>\n");
  print("<textarea name=text rows=40 cols=140 style='font-family: courier new; font-size: 11px;'>$text</textarea>\n");
  print("</td></tr></table>\n");
  print("</form>\n");
  }

sub msg{
  my ($info,$msg)=@_;
  my $color='#E0A0A0';
  $color='#A0E0A0' if($info); 
  print("<table width='100%' border=1 bgcolor='$color'><tr><td><b><center>$msg</center></b></td></tr></table>\n");
  }

#===========================================================================================
# Body

print("Content-type: text/html\n\n");

# Load caboodle configuration file

$cabEnv=\%cabEnv;
my $configName=$ENV{'NB_WEBSTER_CONFIG'};
$configName='etc/CaboodleConfig.conf' if(!$configName);
$msg=Caboodle::loadConfig($cabEnv,$configName);  # load configuration
if($msg){abort($msg);}
$settings=param('settings');      # see if the user wants to use alternate settings
$userEnv=Caboodle::setUser($settings);     # load user settings
my $cabTitle=$cabEnv{Title};
$cabTitle='Temitor' if(!$cabTitle);
my $cabVersion=$cabEnv{Version};
$cabVersion='Release 0.8.0' if(!$cabVersion);
my $cabLink=$cabEnv{Link};
$cabLink='http://www.nodebrain.org' if(!$cabLink);
my $cabMenu=$cabEnv{Menu};
$cabMenu="<a href='/planner/nbkPlanner.cgi'>Planner</a>" if(!$cabMenu);
$cabCopyright=$cabEnv{Copyright};
$cabCopyright='Copyright &copy; 1999-2011 The Boeing Company' if(!$cabCopyright);

my $pageTitle="$cabTitle Planner";
#  var helpWindow=window.open('nbhelp.cgi?'+page,'NodeBrainPlannerHelp','height=640,width=420,location=no,status=no');
my $pageScript="<script language='JavaScript' type='text/javascript'>
<!--
function help(page){
  var helpWindow=window.open('nbhelp.cgi?'+page,'NodeBrainPlannerHelp','height=640,width=420,location=0,dialog=1');
  helpWindow.focus();
  }
//-->
</script>
";
Caboodle::displayPageHead($cabEnv,$pageTitle,$pageScript);  # display page heading now that we have settings and title
chop($pwd=`pwd`);
chop($hostname=`hostname`);
my ($fqdn)=gethostbyname($hostname);
$hostname=$fqdn if($fqdn);
print("
<table width='100%' cellspacing=0 border=0 cellpadding=0 bgcolor='#000000'>
<tr><td width='46' valign='middle' bgcolor='#000000'>
<table cellpadding='0' bgcolor='#000000'>
<tr><td colspan=2><span style='font-size: 2px;'>&nbsp;</span></td></tr>
<tr><td><span style='font-size: 6px;'>&nbsp;</span></td><td valign='middle'><img src='/nb32.gif' height='32' width='32'></td></tr>
<tr><td colspan=2><span style='font-size: 2px;'>&nbsp;</span></td></tr>
</table>
</td>
<td valign='middle'>
<table cellpadding=0 cellspacing=0>
<tr>
<td><span style='font-size: 21px; font-family: arial, sans-serif; color: white'>NodeBrain</td>
<td valign='top'><span style='font-size: 8px; color: white'>TM</td>
<td valign='bottom'><span style='font-size: 16px; color: white'>Planner<span></td>
</tr>
</table>
</td>
<td align='center' valign='middle'>
<a class='plain' href='$cabLink'>
<span class='cabTitle'>$cabTitle<span></a>
<span class='cabVersion'>$cabVersion<span>
</td>
<td align='center' valign='middle'>
<span style='font-size: 10px; color: white'>
C a b o o d l e &nbsp; K i t &nbsp; 0.8.14
</span>
<span style='font-size: 3px; color: white'><br><br></span>
<span style='font-size: 10px; color: white'>
$hostname:$pwd
</span>
</td>
</tr>
<tr><td class='navbar' colspan=3>&nbsp;</td>
<td class='navbar' colspan=4 align='center'>$cabMenu</td></tr>
<tr><td height=1 colspan=4 bgcolor='#000000'/></tr>
</table>
");

# This is the part we are experimenting with

$parser=new XML::Parser(ErrorContext => 2, Style => 'Tree');
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=gmtime();
$time=sprintf("%4.4u/%2.2u/%2.2u %2.2u:%2.2u:%2.2u GMT",$year+1900,$mon+1,$mday,$hour,$min,$sec);
$date=sprintf("%4.4u/%2.2u/%2.2u",$year+1900,$mon+1,$mday);
$userid=$$userEnv{Id};
$name=$$userEnv{Email};
$email=$$userEnv{Email};

$planner=new Planner($userid,$email); # Get a planner object to pass to scheme objects

# Load table definition and create update form

$request=param('request');
$path=param('path');
$browser=$ENV{'HTTP_USER_AGENT'};
#msg(0,"This interface does not work with Microsoft Internet Explorer.  Please use FireFox instead.") if($browser=~/MSIE/);


if(!$request){
  $request='Display';
  $path='_Home';
  $filePath=$path;
  $element='';
  }
else{
  ($filePath,$element)=split(/:/,$path);
  ($filePath,$element)=($1,$2) if(!$element && $filePath=~/^(.*?)\.(.*)$/);
  }

my $planPart=$filePath;
$planPart=$2 if($planPart=~/(.*?\/)+([^\/]+)$/);
my $plan=$planPart;
$plan=$1 if($plan=~/^(.*?)~$/);

abort("Requested plan '$plan' not found.") if(! -f "plan/$plan/$plan.xml");
abort("Requested plan '$plan' control file not found.") if(! -f "plan/$plan/$plan~.xml");

if($request eq 'Edit' || $request eq 'Display'){
  $action='Display';
  $action='Edit' if($request eq 'Edit');
  #my $planObj=new Plan($planner,$filePath);
  my $planObj=new Plan($planner,$path);
  my $html=$planObj->html($filePath,$element,$action);
  print($html);
  }
elsif($request eq 'Update'){
  my %parent;
  my $modified=0;
  $action=param('action');
  $action=param('verb') if(!$action);
  my $planObj=new Plan($planner,$path);
  if($planObj && $planObj->{CONTROL}->getAuthorized($action)){
    if($action eq 'AddParent'){
      my $relative=param('relative');
      my $msg=Config::setRelationState($plan,'enabled',$relative,'parent');
      if($msg){msg(0,$msg);}
      else{
        msg(1,"$relative plan identified as parent of $plan plan.");
        $parent{$relative}=1;
        }
      }
    elsif($action eq 'AddChild'){
      my $relative=param('relative');
      my $msg=Config::setRelationState($plan,'enabled',$relative,'child');
      if($msg){msg(0,$msg)}
      else{
        msg(1,"$relative plan identified as child of $plan plan.");
        $modified=1;
        }
      }
    elsif($action eq 'ToggleRelative'){
      foreach my $relationship ('Child','Parent','Self'){
        foreach my $verb ('enable','disable','resolve'){
          my @name=$planner->getParams("$verb$relationship");
          my $relation=lc($relationship);
          foreach my $name (@name){
            my $msg=Config::setRelationState($plan,$verb.'d',$name,$relation);
            if($msg){msg(0,$msg);}
            else{
              msg(1,"$name plan ${verb}d as $relation of $plan plan.");
              if($relationship eq 'parent'){$parent{$name}=1;}
              else{$modified=1;}
              }
            }
          }
        }
      }
    elsif($action eq 'DeleteRelative'){
      foreach my $relationship ('Child','Parent','Self'){
        foreach my $verb ('enable','disable','resolve'){
          #print("plan=$plan relationship=$relationship verb=$verb<br>\n");
          my @name=$planner->getParams("$verb$relationship");
          my $relation=lc($relationship);
          foreach my $name (@name){
            #print("plan=$plan name=$name relation=$relation<br>\n");
            my $msg=Config::setRelationState($plan,'removed',$name,$relation);
            if($msg){msg(0,$msg);}
            elsif($relation eq 'self'){
              msg(1,"$name plan removed.");
              my @qualifier=split(/\//,$filePath);
              $filePath='';
              foreach my $qualifier (@qualifier){
                $filePath.='/'.$qualifier if($qualifier ne $name);
                }
              $filePath=substr($filePath,1);
              }
            else{
              msg(1,"$name plan removed as $relation of $plan plan.");
              if($relation eq 'child'){$modified=1;}
              else{$parent{$name}=1;}
              }
            }
          }
        }
      }
    elsif($element=~/^Folder/){
      my $canEdit=$planObj->{CONTROL}->getAuthorized('Edit',0);
      my $canCntl=$planObj->{CONTROL}->getAuthorized('Edit',1);
      my $folderObj=$planObj->{FOLDER};
      my $msg=$folderObj->transact($filePath,$element,$action,$canEdit,$canCntl);
      msg(0,$msg) if($msg);
      }
    else{
      my $planXml=$planObj->xml($filePath,$element,$action);
      if($planXml && $planXml=~/^</){   
        $planFile="plan/$plan/$planPart.xml";
        $bakSuffix=sprintf(".%10.10d",int(time()/(60*60))*60*60);  # only backup once within a given wall clock hour
        $planFileBak=$planFile.$bakSuffix;
        system("cp $planFile $planFileBak") if(! -f "$planFileBak");
        if(open(PLAN,">$planFile")){  # store updated plan file
          #$planXml=$planner->encodeAmp($planXml); # encode ampersand for now until all xml methods encode data
          print(PLAN $planXml);
          close(PLAN);
          msg(1,"Plan XML file updated successfully.");
          $modified=1;
          }
        else{msg(0,"Unable to open $planFile for output.");}
        }
      else{msg(0,$planXml);} # display message returned
      $planObj=new Plan($planner,$path);
      }
    $planner->alertChange();
    if($modified){
      if(-d "plan/$plan"){
        my $msg=Config::compile($plan);
        if($msg){msg(0,$msg);}
        elsif($msg eq ''){msg(1,"$plan plan does not require compile.");}
        else{msg(1,"$plan plan compiled successfully.");}
        }
      }
    foreach my $plan (sort keys %parent){
      if(-d "plan/$plan"){
        my $msg=Config::compile($plan);
        if($msg){msg(0,$msg);}
        elsif($msg eq ''){msg(1,"$plan plan does not require compile.");}
        else{msg(1,"$plan plan compiled successfully.");}
        }
      }
    }
  my $html;
  if(-d "plan/$plan"){
    $html=$planObj->html($filePath,$element,'Edit');
    }
  else{
    $planObj=new Plan($planner,"Home");
    $html=$planObj->html('Home','','Display');
    }
  print($html);
  }
elsif($request eq 'Code'){
  if(open(INDEX,"plan/$plan/$plan.index")){
    while(<INDEX>){
      next if(/^#/);
      if(/^(.*?)( (.*))?$/){
        my $file=$1;
        my $title=$3;
        print("<table border=0 cellspacing=0 cellpadding=0 width='100%'><tr><th>&nbsp; $file &nbsp; $title</th></tr></table>\n<pre>");
          if(open(FILE,"$file")){
          print($planner->encode($_)) while(<FILE>); 
          close(FILE);
          }
        else{print("*** unable to open $file ***\n");}
        print("</pre>\n");
        }
      }
    }
  else{
    my $nb='';
    if(!open(NB,"plan/$plan/$plan.nb")){
      msg(0,"Unable to open code file.");
      }
    $nb.=$_ while(<NB>);
    close(NB);
    $nb=$planner->encode($nb);
    print("<pre>$nb</pre>\n");
    }
  }
elsif($request eq 'Config'){
  my $planObj=new Plan($planner,$path);
  my $cfg=$planObj->cfg();
  $cfg=$planner->encode($cfg);
  print("<pre>$cfg</pre>\n");
  }
elsif($request eq 'Table'){
  my $planObj=new Plan($planner,$path);
  my $table=$planObj->table();
  $table=$planner->encode($table);
  print("<pre>$table</pre>\n");
  }
elsif($request eq 'Help'){
  my $ht='';
  if(!open(HT,"plan/$plan/$plan.help")){
    my $planObj=new Plan($planner,$path);
    if($planObj){
      #my $planScheme=
      if(!open(HT,"plan/$plan/$plan.help")){msg(0,"Unable to open help file for $plan plan.");}
      }
    else{msg(0,"Unable to open help file for $plan plan.");}
    }
  $ht.=$_ while(<HT>);
  close(HT);
  print("$ht");
  }
else{msg(0,"Request '$request' not recognized.");}

print("<span class=foot><hr>$cabCopyright</span></body></html>\n");
