# NodeBrain Planner 0.8.0 (XML Schema 0.8)
#
# Name:      CaboodleAlarmer
#
# Title:     Alarm Flood Protection
#
# Purpose:
#   
#   This plan is used by agents to generate alarms with flood control
#
# Warning:
#
#   Do not edit this file manually unless it is no longer under the control of NodeBrain Planner.
#   Otherwise, your changes will be removed by the next update.
#
# Path:      CABOODLE/plan/CaboodleAlarmer
#
# Scheme:    _Block
#
#================================================================================================
# Permits:
#
#   Administrator (owner) <root@localhost>
#
#================================================================================================
# History:
#
# Date       Time     User - Change - Reason
# ---------- -------- ------------------------------------------------------------------------------
# 2013-05-04 23:27:50 Administrator - Full plan update - 
#================================================================================================

# Block 1  

# 2004/03/08 eat - NodeBrain 0.6.0 syntax
# 2005/09/19 eat 0.5.0  included facts and url
# 2013-04-14 eat 0.8.14 modified for Caboodle Kit
#===================================================================
%default \
  source="Caboodle",\
  appl="Caboodle",\
  reset="8h",\
  interval="1h",\
  hits="^50,100,200,400",\
  kids="5,10,20";

# Block 2  

% define alarm node cache(~(%{reset})):(~(%{interval}):msgid(%{hits}));
alarm. define route    cell; # Routing codes
alarm. define group    cell; # General topic classifier
alarm. define severity cell; # Level of importance
alarm. define text     cell; # Messages text
alarm. define facts    cell; # List of facts for indexing
alarm. define url      cell; # URL for drill down link

% alarm. define r0 if(msgid._hitState="normal")\
  :$ =|bin/nbkit_alarm alarm source="%{source}" appl="%{appl}" \
    route="${route}" \
    group="${group}" \
    severity="${severity}" \
    facts="${facts}" \
    url="${url}" \
    text="${msgid} A[${msgid._hits}] ${text}"

% alarm. define r1 if(msgid._hitState<>"normal")\
  :$ =|bin/nbkit_alarm alarm source="%{source}" appl="%{appl}" \
    route="${route}" \
    group="${group}" \
    severity="critical" \
    text="${msgid} ${msgid._hits} of these alarms over ${alarm._interval} - suppressing flood";

alarm. define r2 if(1)[1] facts="",url="";   # clean up optional values

# Block 3  

% define alarmS node cache(~(%{reset})):(~(%{interval}):msgid,summary[%{kids}],detail(2));
alarmS. define route     cell;
alarmS. define group     cell;
alarmS. define severity  cell;
alarmS. define text      cell;
alarmS. define facts     cell;
alarmS. define url       cell;
# Parameters used for summary alarms - these promote to a non-S parameter when we summarize
alarmS. define severityS cell;
alarmS. define factsS    cell;
alarmS. define urlS      cell;

% alarmS. define r0 if(detail._hitState="normal" and summary._kidState="normal")\
  :$ =|bin/nbkit_alarm alarm source=%{source} route="${route}" appl="%{appl}" group="${group}" \
  severity="${severity}" facts="${facts}" url="${url}" text="${msgid} B[${summary._kids}] ${text}"

alarmS. define r1 if(detail._hitState="normal" and summary._kidState<>"normal"):$ alarm. \
  alert ("${msgid}"),route="${route}",group="${group}",severity="${severityS}",facts="${factsS}",\
  url="${urlS}",text="${summary._kids} ${summary} in ${alarmS._interval}"

alarmS. define r2 if(1)[1] factsS="",facts="",urlS="",url="";   # clean up optional values

# Block 4  

% define alarmSS node cache(~(%{reset})):(~(%{interval}):msgid,summary[%{kids}],detail(2));
alarmSS. define route      cell;
alarmSS. define group      cell;
alarmSS. define severity   cell;
alarmSS. define text       cell;
alarmSS. define facts      cell;
alarmSS. define url        cell;
# Parameters used for summary alarms - these promote to a non-S parameter when we summarize
alarmSS. define severityS  cell;
alarmSS. define factsS     cell;
alarmSS. define urlS       cell;
# Parameters used for second level summary alarms - promote to S parameters
alarmSS. define textSS     cell;
alarmSS. define detailSS   cell;
alarmSS. define severitySS cell;
alarmSS. define factsSS    cell;
alarmSS. define urlSS      cell;

% alarmSS. define r0 if(detail._hitState="normal" and summary._kidState="normal")\
  :$ =|bin/nbkit_alarm alarm source=%{source} route="${route}" appl="%{appl}" group="${group}" \
  severity="${severity}" facts="${facts}" url="${url}" text="${msgid} C[${summary._kids}] ${text}";

alarmSS. define r1 if(detail._hitState="normal" and summary._kidState<>"normal"):$ $ alarmS. \
  alert ("${msgid}","${textSS}","${detailSS}"),route="${route}",group="${group}",\
  severity="${severityS}",severityS="${severitySS}",\
  factsS="${factsSS}",facts="${factsS}",urlS="${urlSS}",url="${urlS}",\
  text="${summary._kids} ${summary} in ${alarmSS._interval}";

# clean up optional values
alarmSS. define r2 if(1)[1] factsSS="",factsS="",facts="",urlSS="",urlS="",url="";

#========================================================================
# Children:
#========================================================================

